﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectBulldozer.Drawnings;
using ProjectBulldozer.MovementStrategy;

namespace ProjectBulldozer;

/// <summary>
/// Форма работы с объектом "Бульдозер"
/// </summary>
public partial class FormBulldozer : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningDozer? _drawningDozer;

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    /// <summary>
    /// Получение объекта
    /// </summary>
    public DrawningDozer SetCar
    {
        set
        {
            _drawningDozer = value;
            _drawningDozer.SetPictureSize(pictureBoxBulldozer.Width, pictureBoxBulldozer.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormBulldozer()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки транспорта
    /// </summary>
    private void Draw()
    {
        if (_drawningDozer == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxBulldozer.Width, pictureBoxBulldozer.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningDozer.DrawTransport(gr);
        pictureBoxBulldozer.Image = bmp;
    }

    /*/// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningDozer):
                _drawningDozer = new DrawningDozer(random.Next(100, 300), random.Next(1000, 3000),
            Color.FromArgb(random.Next(170, 256), random.Next(170, 256), random.Next(30, 140)),
            Color.FromArgb(random.Next(30, 120), random.Next(30, 120), random.Next(30, 120)));
                break;
            case nameof(DrawningBulldozer):
                _drawningDozer = new DrawningBulldozer(random.Next(100, 300), random.Next(1000, 3000),
            Color.FromArgb(random.Next(170, 256), random.Next(170, 256), random.Next(30, 140)),
            Color.FromArgb(random.Next(30, 120), random.Next(30, 120), random.Next(30, 120)),
            Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningDozer.SetPictureSize(pictureBoxBulldozer.Width, pictureBoxBulldozer.Height);
        _drawningDozer.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    /// <summary>
    /// Обработка нажатия кнопки "Создать крутой бульдозер"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateBulldozer_Click(object sender, EventArgs e) =>
        CreateObject(nameof(DrawningBulldozer));

    /// <summary>
    /// Обработка нажатия кнопки "Создать бульдозер"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateDozer_Click(object sender, EventArgs e) =>
        CreateObject(nameof(DrawningDozer));
*/

    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningDozer == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningDozer.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningDozer.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningDozer.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningDozer.MoveTransport(DirectionType.Right);
                break;
        }
        if (result)
        {
            Draw();
        }
    }

    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningDozer == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorderLB(),
                2 => new MoveToBorderRB(),
                _ => null,
            };

            if (_strategy == null)
            {
                return;
            }

            _strategy.SetData(new MoveableDozer(_drawningDozer), pictureBoxBulldozer.Width,
                pictureBoxBulldozer.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}
